import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.border.*;
 
class JDiagram extends JPanel 
	implements 	MouseListener, 
				MouseMotionListener,
				MouseWheelListener   {
 
	JFiraFont fira = JFiraFont.getInstance();
 
	private JBinomial parent;
 
	private final static int mx = 60;
	private final static int my = 40;

	private final int[] arrowPx = new int[] {mx, mx-10, mx+10 };
	private final int[] arrowPy = new int[] { 0, mx/2,  mx/2  };
 
	private int hj, wj;
    private int kmin, kmax;

	private final Color myBlack = new Color(10,10,10);
	private final Color myGray  = new Color(200,200,200);
	private final Color myWhite = new Color(250,250,250);

    private final Stroke thick1 = new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    private final Stroke thick2 = new BasicStroke(2.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    private final Stroke thick3 = new BasicStroke(3.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
 
	
	private Double pmax, psteps, pi, pmin;

	private Font axFont, smallAxFont, resultFont;
	private FontMetrics fm;
	private String piStr;
	private String pString = String.format( "%.1f", 0.5 ) ;
	
	public JBinomTable binomTable;

	private int ksum, kanz, mymod, py1, py2, pb, rx1, rx2, kx;

	public JPopup 		popup;
	public JShowTable 	showTable;


	private Boolean 	show = false;
	private int 		showMode = 0;
	private int 		showK1 = 5;
	private int 		showK2 = 15;

	private Double		showSigmaA = 1.0;

	private int 		hypoMode = 0;
	private Double		showHypoAlpha = 0.05;
	private Double		showHypoAlphaStrich;

 
    public JDiagram(JBinomial p) {
		this(p, 20, 0.5);
	}
	
    public JDiagram(JBinomial p, int nn, Double pp) {

		parent = p;

        setBackground(myWhite);

//		axFont      = new Font("sansserif", Font.BOLD, 18);
//		smallAxFont = new Font("sansserif", Font.BOLD, 16);
		axFont      = fira.font24;
		smallAxFont = fira.font22;

//		resultFont  = new Font("sansserif", Font.BOLD, 20);
		resultFont  = fira.font26;

		binomTable  = new JBinomTable(nn, pp);
		
		pmax = calcPmax();
		
		kmin = 0; kmax = nn;
		
		addMouseListener(this);
		addMouseMotionListener(this);
		addMouseWheelListener(this);
		
		popup = new JPopup(p);
		setComponentPopupMenu(popup);

		showTable = new JShowTable( binomTable );
		
		showK1 = nn/2; showK2 = nn;
	
	}

	public void updateP( Double pp, String ps ) {
	
		binomTable.reCalcP( pp );
		pString = ps;
		repaint();
		
	}

	public void updateN( int nn ) {
	
		binomTable.reCalcN( nn );
		repaint();
		
	}

	public void setFull() {

		kmin = 0; kmax = binomTable.getN();	pmax = 1.0;

		parent.bottomBar.setMinMax( kmin, kmax );
				
		repaint();
		
	}

	public void setOptimum() {

		pmax = calcPmax();

		pmin = 1.0 * pmax / (hj-2*my);
		
		int kstart = (int)Math.round( binomTable.getN()*binomTable.getP() );
		
		kmin = kstart;
		
		while ( (kmin > 0) && 
		        ( binomTable.getSingle(kmin) >= pmin )
		      ) { kmin--; }

		kmax = kstart;
		
		while ( ( kmax < binomTable.getN() ) && 
		        ( binomTable.getSingle(kmax) >= pmin )
		      ) { kmax++; }

		parent.bottomBar.setMinMax( kmin, kmax );

		repaint();
	}

	
    @Override
    public void paintComponent(Graphics gg) {
        super.paintComponent(gg);
        Graphics2D g = (Graphics2D) gg;

		paintDiagram(g);

    }
 
	private void paintDiagram(Graphics2D g) {

        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
 
		hj = getHeight(); wj = getWidth();

		g.setColor(myBlack);

		ksum = wj - 2*mx; kanz = kmax - kmin + 1;
		
		drawAxes(g);
		
		if ( show && ( showMode == 3 ) ) { showMuSigma( g ); }

		if ( show && ( showMode == 4 ) ) { calculateHypo(); }
		
		if ( kanz < 10 ) { g.setStroke(thick3); mymod = 1; }
		  else if ( kanz < 20 ) { g.setStroke(thick2);  mymod = 2; }
		         else { g.setStroke(thick1);
					    if ( kanz < 50 ) {mymod = 5;} else {mymod = 10;} 
					  }

		g.setFont(axFont); fm = g.getFontMetrics();

		pb = hj-2*my;

		for (int i=0; i<kanz; i++) {

			rx1 = mx + (i*ksum)/kanz;
			rx2 = mx + ((i+1)*ksum)/kanz;

			if ( i == 0 ) { g.drawLine(rx1, hj-my, rx1, hj-my+8 ); }
			g.drawLine(rx2, hj-my, rx2, hj-my+8 );

			
			py1 = (int)Math.round( (binomTable.getSingle(kmin+i)*pb)/pmax );
			py2 = hj-my-py1;

			if ( py1 > 0 ) {

				if ( show ) {
					g.setColor( getCurrentColor(kmin+i) );
				} else {
					g.setColor( Color.green );
				}

				g.fillRect(rx1, py2 , rx2-rx1, py1);
				g.setColor(myBlack);
				g.drawRect(rx1, py2, rx2-rx1, py1);

			}

			g.setColor( myBlack );

			if ( (kmin+i) % mymod == 0) {

			String kstr = ""+(kmin+i);
			kx = rx1 + (rx2 - rx1 - fm.stringWidth(kstr) ) / 2;
			
			g.drawString(kstr, kx, hj-my/2  + (my - fm.getAscent())/2  );
			}

		}
		
		if ( show ) { showResult(g); }

	} 


	private void showResult(Graphics2D g) {

		String 	result="";
		int 	resWidth;
		
		switch( showMode ) {
			
		case 0 : 	result = "P ( X = "+showK1+" ) = B ( " + 
		                     binomTable.getN() + "; " + pString + "; " +
		                     showK1 + " ) = " + 
		                     String.format("%.4f", binomTable.getSingle( showK1) );
					break;

		case 1 : 	result = "P ( X ≤ "+showK1+" ) = F ( " + 
		                     binomTable.getN() + "; " + pString + "; " +
		                     showK1 + " ) = " + 
		                     String.format("%.4f", binomTable.getCumulated( showK1) );
					break;

		case 2 : 	if ( showK1 == 0 ) {
						result = String.format("%.4f", binomTable.getCumulated(showK2) );
					} else {
						result = String.format("%.4f",  binomTable.getCumulated(showK2) -
														binomTable.getCumulated(showK1-1));
					}

					result = "P ( "+showK1+" ≤ X ≤ "+showK2+" ) = " + result;
					break;

		case 3 : 	result = String.format("%.2f", showSigmaA );

					if ( result.matches("[0-9][\\.,][1-9]0$") ) { 
						result = result.substring(0, result.length()-1 );
					} else if ( result.matches("[0-9]+[\\.,]00$") ) { 
						result = result.substring(0, result.length()-3 );
					}
		
					if ( result.equals("1") ) { result=""; }
					                     else { result = result+" "; }

					result = "P ( μ - "+result+"δ ≤ X ≤ μ + "+result+"δ ) ≈ " + 
		                     String.format("%.4f", binomTable.getInterval( showK1, showK2 ) );
					break;

		case 4 : 	switch( hypoMode) { 
			
					case 0: result = "A = ["+showK1+".."+showK2+"]   "+
									 "A' = [0.."+(showK1-1)+"] U ["+showK2+".." + 
									 binomTable.getN()+"]";

							break;

					case 1: result = "A = ["+showK1+".."+showK2+"]   "+
									 "A' = [0.."+(showK1-1)+"]";

							break;

					case 2: result = "A = ["+showK1+".."+showK2+"]   "+
									 "A' = ["+(showK2+1)+".." + 
									 binomTable.getN()+"]";

							break;
					}

					result = result + "   α' = " +
							 String.format( "%.2f", showHypoAlphaStrich*100)+"%";

					break;
		}

		g.setFont( resultFont );
		fm = g.getFontMetrics();
		resWidth= fm.stringWidth( result );

		g.setColor(Color.blue);
		g.drawString( result, mx + (wj-2*mx-resWidth)/2, my-5 );

	}

 
	private void drawAxes(Graphics2D g) {

		g.setStroke(thick3);
		g.setColor( myBlack );

		g.drawLine(mx, hj-my, wj-5, hj-my);
		g.drawLine(mx, hj-my, mx, 5);

		g.fillPolygon( new int[] {wj, wj-mx/2, wj-mx/2},
					   new int[] {hj-my, hj-3*my/4, hj-(5*my)/4},
					   3);

		g.fillPolygon( arrowPx, arrowPy, 3);

		// labels

		g.setFont(axFont); fm = g.getFontMetrics();

		g.drawString("k", wj-mx/2, hj-(3*my)/2 );
		g.drawString("p", mx/4, (2*my)/3 );

		// y axis

		Double psteps = 0.0;
		
		       if ( pmax <= 0.01 )  { psteps = 0.001; }
		  else if ( pmax < 0.05 )  { psteps = 0.005; }
		  else if ( pmax < 0.10 )  { psteps = 0.010;  }   
		  else if ( pmax < 0.20 )  { psteps = 0.020; }   
		  else if ( pmax < 0.50 )  { psteps = 0.050; }   
							  else { psteps = 0.100; }   

		Double pi = psteps;

		g.setFont( smallAxFont); fm = g.getFontMetrics();
		
		while ( pi <= pmax ) {

			int py = hj - my - (int)Math.round( ((hj-2*my)*pi)/pmax);

			g.setColor(myBlack); g.drawLine( (4*mx)/5, py , mx, py );
			g.setColor(myGray);  g.drawLine( mx+2, py, wj-mx, py );
			
			piStr=String.format("%.2f",pi);

			g.setColor(myBlack);
			g.drawString( piStr, (7*mx)/10 - fm.stringWidth(piStr),
								  py + fm.getAscent()/2 );

			pi = pi + psteps;
		}


	}

	private void drawArrow(Graphics2D g, double theta, double x0, double y0) {
        
		int barb = 15;
		Double phi = Math.PI/6;
			
		double x = x0 - barb * Math.cos(theta + phi);
		double y = y0 - barb * Math.sin(theta + phi);
		g.draw(new Line2D.Double(x0, y0, x, y));
		x = x0 - barb * Math.cos(theta - phi);
		y = y0 - barb * Math.sin(theta - phi);
		g.draw(new Line2D.Double(x0, y0, x, y));
	}

	private void showMuSigma( Graphics2D g ) {

		Double dkMu = (1.0*(wj-2*mx))/(kmax-kmin);
		Double wkMu = 1.0*wj-2*mx-dkMu;

		Double xkMu = wkMu*( binomTable.getMu()-kmin )/(kmax-kmin) +
		              mx + 0.5*dkMu;
		
		Double kS1  = binomTable.getMu() - showSigmaA*binomTable.getSigma();
		Double kS2  = binomTable.getMu() + showSigmaA*binomTable.getSigma();

		Double xkS1 = wkMu*( kS1 - kmin )/(kmax-kmin) + mx + 0.5*dkMu;
		Double xkS2 = wkMu*( kS2 - kmin )/(kmax-kmin) + mx + 0.5*dkMu;

		g.setClip( new Rectangle(mx, 0, wj-2*mx, hj) );

		g.setColor( new Color(0f, 0f, 1f, 0.1f ) );
		g.fillRect( (int)Math.round( xkS1 ), 2*my,
					(int)Math.round( xkS2 ) - (int)Math.round( xkS1 ), hj-3*my);
		
		g.setColor( Color.blue );
		g.setStroke( thick3 );
		
		int myt1 = (int)Math.round( (3.0*my) /2);
		int myt2 = (int)Math.round( (5.0*my) /2);
		
		g.drawLine( (int)Math.round( xkMu ), 2*my,
					(int)Math.round( xkMu ), hj-my );

		g.drawLine( (int)Math.round( xkS1 ), myt1,
					(int)Math.round( xkS1 ), hj-my  );

		g.drawLine( (int)Math.round( xkS2 ), myt1,
					(int)Math.round( xkS2 ), hj-my  );

		g.drawLine( (int)Math.round( xkS1 ), 2*my,
					(int)Math.round( xkS2 ), 2*my  );

		drawArrow(g, Math.PI, xkS1, 2.0*my);
		drawArrow(g, 0.0, xkS2, 2.0*my);

		g.setClip( null );
					
		if ( Math.floor(kS1) == kS1 ) {	showK1 = (int)Math.floor(kS1); } else {	showK1 = (int)Math.floor(kS1) + 1; }
		showK2 = (int)Math.floor(kS2);

	}

	private void calculateHypo() {

		showK1 = 0; showK2 = binomTable.getN();
		
		switch ( hypoMode ) {

		case 0: while ( binomTable.getCumulated( showK1 ) < showHypoAlpha/2 ) { showK1++; }
				while ( 1 - binomTable.getCumulated( showK2 ) < showHypoAlpha/2 ) { showK2--; }
				showK2++;
				showHypoAlphaStrich = 1.0 - ( binomTable.getCumulated(showK2) - binomTable.getCumulated(showK1-1) );
				break;

		case 1: while ( binomTable.getCumulated( showK1 ) < showHypoAlpha ) { showK1++; }
				showHypoAlphaStrich = binomTable.getCumulated(showK1-1);
				break;

		case 2: while ( 1 - binomTable.getCumulated( showK2 ) < showHypoAlpha ) { showK2--;	}
				showK2++;
				showHypoAlphaStrich = 1.0 - binomTable.getCumulated(showK2);
				break;

		}

	}
	
	private Double calcPmax() {

		int mu = (int)( binomTable.getN()*binomTable.getP() );
		
		if ( binomTable.getSingle(mu+1) > binomTable.getSingle(mu) ) {
			return 1.2*binomTable.getSingle(mu+1);
		} else {
			return 1.2*binomTable.getSingle(mu);
		}

	}

	// bar colors...
	//-------------------------------------------------------------------

	private Color getCurrentColor(int kk) {

		Color aktColor = Color.green;

		switch ( showMode ) {

		case 0 : if ( kk == showK1 ) { aktColor =  Color.red;   }
				 break;

		case 1 : if ( kk <= showK1 ) { aktColor =  Color.red;   }
				 break;

		case 2: 
		case 3:  if ( (kk >= showK1) && (kk <= showK2) ) { aktColor =  Color.red;   }
				 break;

		case 4:  if ( (kk < showK1) || (kk > showK2) ) { aktColor =  Color.red;   }
				 break;

		}
		
		return aktColor;
	}


	// show controlling...
	//-------------------------------------------------------------------

	public void switchShow( Boolean sw ) {
		show = sw;
		repaint();
	}

	public void setShowMode( int m ) {
		showMode = m;

		if ( m <= 1 ) {
			showK1 = parent.bottomBar.singleKslider.getValue();
		}

		if ( m == 2 ) {
			showK1 = parent.bottomBar.getIntervalSliderMin();
			showK2 = parent.bottomBar.getIntervalSliderMax();
		}

		repaint();
	}

	public void setShowKValues( int k1, int k2 ) {
		if ( showMode <= 2 ) {
			showK1 = k1;
			showK2 = k2;
			repaint();
		}
	}

	public void setShowSigmaA( Double a ) {

		if ( showMode == 3 ) {
			showSigmaA = a;
			repaint();
		}

	}

	public void setHypoMode( int m ) {

		hypoMode = m;
		repaint();

	}

	public void setShowHypoAlpha( Double a ) {

		if ( showMode == 4 ) {
			showHypoAlpha = a;
			repaint();
		}

	}

	// mouse controlling...
	//-------------------------------------------------------------------

	static final int xMode_LEFT   = 0;
	static final int xMode_MIDDLE = 1;
	static final int xMode_RIGHT  = 2;

	private Boolean	mouseaction = false;
	private Point	startPoint = null;
	private Point	endPoint = null;
	private int 	dheight, dwidth, dx, dy, xMode, kDelta;
	private Double 	actPmax, newPmax;

	public void mouseEntered(MouseEvent e)	{	}
	public void mouseExited(MouseEvent e)	{	}
	public void mouseClicked(MouseEvent e)	{	}
	public void mouseMoved(MouseEvent e)	{	}

	public void mousePressed(MouseEvent e)	{
		if ( e.getButton() == 1 ) {

		mouseaction = true;
		startPoint 	= e.getPoint();
		endPoint 	= startPoint;
		actPmax 	= pmax;
		
		dheight		= hj-2*my;
		dwidth		= wj-2*mx;

		if ( startPoint.x < mx+dwidth/3 ) { xMode = xMode_LEFT; }
		else if ( startPoint.x > mx+(2*dwidth/3) ) { xMode = xMode_RIGHT; }
		else { xMode = xMode_MIDDLE; }

		}   // if Button=1
	}

	public void mouseDragged(MouseEvent e)	{

		if ( mouseaction ) {
			
			endPoint 	= e.getPoint();
			dx 			= endPoint.x - startPoint.x;
			dy			= endPoint.y - startPoint.y;

			if ( dx == 0 || (1.0*Math.abs(dy))/Math.abs(dx) > 1.2 ) {
				newPmax = actPmax * ( 1 + (1.0*dy)/dheight );
				mouseSetP();
			}			

			if ( dy == 0 || (1.0*Math.abs(dx))/Math.abs(dy) > 1.2 ) {

				kDelta = (int)Math.round( (1.0*dx)*(kmax-kmin)/dwidth);

				if ( kDelta != 0 ) {

					switch ( xMode ) {
					case xMode_MIDDLE:
				
						if ( (kDelta < 0) && ( kmax - kDelta > binomTable.getN() ) ) {
								kDelta = kmax - binomTable.getN();
						}

						if ( (kDelta > 0) && ( kmin - kDelta < 0 ) ) {
								kDelta = kmin;
						}
				
						kmin = kmin - kDelta; kmax = kmax - kDelta; break;

					case xMode_LEFT:
				
						if ( (kDelta < 0) && ( kmin - kDelta >= kmax -1 ) ) {
								kDelta = kmin - kmax + 1 ;
						}

						if ( (kDelta > 0) && ( kmin - kDelta < 0 ) ) {
								kDelta = kmin;
						}
				
						kmin = kmin - kDelta; break;

					case xMode_RIGHT:
				
						if ( (kDelta < 0) && ( kmax - kDelta > binomTable.getN() ) ) {
								kDelta = kmax - binomTable.getN();
						}

						if ( (kDelta > 0) && ( kmax - kDelta <= kmin +1 ) ) {
								kDelta = kmax - kmin - 1;
						}
				
						kmax = kmax - kDelta; break;

					}

					repaint(); startPoint = endPoint;
					
					parent.bottomBar.setMinMax( kmin, kmax );
					
				}

			}			

		}

	}

	public void mouseReleased(MouseEvent e)	{
		if ( e.getButton() == 1 ) {
			startPoint = null; 
			endPoint = null;
			mouseaction = false;
		}
	}
	
 	public void mouseWheelMoved(MouseWheelEvent e) {

		newPmax = pmax * ( 1 + (25.0*e.getWheelRotation())/(hj-2*my) );
		mouseSetP();

	}

	private void mouseSetP() {
		if ( newPmax > 1.0 ) { pmax = 1.0; }
		else if ( newPmax < 0.05 ) { pmax = 0.05; }
		else { pmax = newPmax; }
		repaint();
	}

	// copy to clipboard...
	//-------------------------------------------------------------------

	public void copyToClipboard() {

		Dimension size = getSize();
		BufferedImage myImage = new BufferedImage(size.width, size.height, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = myImage.createGraphics();
		paintDiagram(g2);
		
		JImageSelection imgSel = new JImageSelection(myImage);
		Toolkit.getDefaultToolkit().getSystemClipboard().setContents(imgSel, null);

	}


}
