import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Hashtable;
import javax.swing.border.*;
 
class JSliders extends JReorderPanel {
	
	JBinomial parent;

	JFiraFont fira = JFiraFont.getInstance();

	private final Color myBlack = new Color(10,10,10);
	private final Color myGray  = new Color(200,200,200);
	private final Color myWhite = new Color(250,250,250);

    private final Stroke thick1 = new BasicStroke(1.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    private final Stroke thick2 = new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    private final Stroke thick3 = new BasicStroke(2.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);

	private Font axFont, smallAxFont;
	
	JSliderCombo	pBox, nBox;
	JPSlider 		pSlider;
	JNSlider 		nSlider;

	Double			pValue;
	int				nValue;
 
	Boolean			setPfromBox = false;
	Boolean			setNfromBox = false;

	JPanel 			pPanel, nPanel;

	Component 		dist10 = Box.createRigidArea(new Dimension(10,0));
	
    public JSliders(JBinomial par) {
		this(par, 20, 0.5 );
	}
	
    public JSliders(JBinomial par, int nn, Double pp) {

		parent = par;

		nValue = nn;
		pValue = pp;

		setLayout( new BoxLayout(this, BoxLayout.LINE_AXIS) );
        setBackground(myWhite);
        setBorder(new EmptyBorder(20,10,30,10));

		pPanel = new JPanel(new RelativeLayout(RelativeLayout.Y_AXIS));
        pPanel.setBackground(myWhite);

		pPanel.add( new JSliderLabel("p =    ") ) ; 
		pPanel.add( Box.createRigidArea(new Dimension(0,5)));

		Object[] ppattern = new Object[] {
         "3/4", "2/3", "1/2", "1/3", "1/4", "1/5", "1/6", "1/7", "1/8",
         new JSeparator(JSeparator.HORIZONTAL),
			String.format("%1.1f", 0.9),
			String.format("%1.1f", 0.8),
			String.format("%1.1f", 0.7),
			String.format("%1.1f", 0.6),
			String.format("%1.1f", 0.5),
			String.format("%1.1f", 0.4),
			String.format("%1.1f", 0.3),
			String.format("%1.1f", 0.2),
			String.format("%1.1f", 0.1)
		};

		pBox = new JSliderCombo(parent, null, 
								String.format("%.1f",pp), 
								ppattern, 4);	
		
		pPanel.add( pBox );
		pPanel.add( Box.createRigidArea(new Dimension(0,5)));

		pSlider= new JPSlider(parent, pp);
		pPanel.add( pSlider, new Float(1) );

		add( pPanel );
		
		add( dist10 );

		nPanel = new JPanel(new RelativeLayout(RelativeLayout.Y_AXIS));
        nPanel.setBackground(myWhite);

		nPanel.add( new JSliderLabel("n =    ") ) ; 
		nPanel.add( Box.createRigidArea(new Dimension(0,5)));

		Object[] npattern = new Object[] {
         "100", "90", "80", "75", "70", "60", "50",
          "40", "30", "25", "20", "15", "10",  "5"
		};

		nBox = new JSliderCombo(parent, null, ""+nn, npattern, 3);	
		
		nPanel.add( nBox );
		nPanel.add( Box.createRigidArea(new Dimension(0,5)));

		nSlider= new JNSlider(parent, nn);
		nPanel.add( nSlider, new Float(1) );

		add( nPanel );

	}

	
	public Double getPslider() {
		return 1.0*pSlider.getValue()/1000;
	}

	public String getPsliderString() {

		String s = String.format("%.3f", 1.0*pSlider.getValue()/1000);

		if ( s.matches("[01][\\.,][0-9][1-9]0$") ) { 
			s = s.substring(0, s.length()-1 );
		} else if ( s.matches("[0-9]+[\\.,][0-9]00$") ) { 
		    s = s.substring(0, s.length()-2 );
		}

		return s;
	}

	public void setPslider( Double pp ) {

		pSlider.setValue( (int)Math.round(pp*1000) );

	}


	public Boolean isPboxValid() {

		Boolean isValid = false;
		
		String s = pBox.textField.getText();
		
		if ( s.matches("^ *[1-9][0-9]*/[1-9][0-9]* *$") ) {
			isValid = true;

			int p = s.indexOf("/");
			int z = Integer.parseInt( s.substring(0,p ) );
			int n = Integer.parseInt( s.substring(p+1, s.length()-p+1 ) );
			
			if ( z <= n ) {
				pValue = 1.0*z/n;
			} else {
				isValid = false;
			}
		}

		if ( s.matches("^ *[01][,.][0-9]+ *$") ) {
			pValue = Double.parseDouble( s.replaceAll(",",".") );

			if ( pValue <= 1.0 ) {
				isValid = true;
			}

		};

		if ( s.matches("^ *[1-9][0-9]*[.,]?[0-9]* *% *$") ) {
			System.out.println( s.substring(0,s.indexOf("%")).replaceAll(",",".") );
			pValue = Double.parseDouble( 
			           s.substring(0,s.indexOf("%")).replaceAll(",",".")
			         )/100;

			if ( pValue <= 1.0 ) {
				isValid = true;
			}

		};

		if ( !isValid ) {
			pBox.textField.setForeground(Color.red);
		} else {
			pBox.textField.setForeground(Color.black);
		}

		return isValid;

	}

	public Double getPbox() {
		return pValue;
	}

	public String getPboxString() {
		return pBox.textField.getText();
	}

	public void setPbox() {

		pBox.textField.setForeground(Color.black);
		pBox.textField.setText( String.format("%.3f", 1.0*pSlider.getValue()/1000) );

	}


	public int getNslider() {

		return nSlider.getValue();

	}

	public void setNbox() {

		nBox.textField.setForeground(Color.black);
		nBox.textField.setText( ""+nSlider.getValue() );

	}

	public Boolean isNboxValid() {

		Boolean isValid = false;
		
		String s = nBox.textField.getText();
		
		if ( s.matches("^ *1?[0-9]?[0-9] *$") ) {
			isValid = true;

			nValue = Integer.parseInt( s );
			
			if ( nValue > 100 ) { isValid = false; }
		}

		if ( !isValid ) {
			nBox.textField.setForeground(Color.red);
		} else {
			nBox.textField.setForeground(Color.black);
		}

		return isValid;

	}
 
	public int getNbox() {
		return nValue;
	}

	public void setNslider( int nn ) {

		nSlider.setValue( nn );

	}
 
	//---- class JSLiderLabel
	
	private class JSliderLabel extends JLabel {

		public JSliderLabel(String s) {
			super( s );
			setFont( fira.font22 );
		}

	}

	private class JPSlider extends JWheelSlider {

		public JPSlider(JBinomial parent, Double p) {

			super(JSlider.VERTICAL, 0, 1000, (int)(p*1000));
			setMajorTickSpacing(100);
			setMinorTickSpacing(50);
			setPaintTicks(true);

			Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();

			for (int i=0; i<=10; i++) {
				labelTable.put( new Integer( i*100 ), new JPTickLabel( String.format("%.1f",(1.0*i)/10) ));
			}

			setLabelTable( labelTable );
			setPaintLabels(true);
			addChangeListener(parent);

		}

		private class JPTickLabel extends JLabel {

			public JPTickLabel(String s) {
				super( s );
				setFont( fira.font22 );
			}

		}

	}

	private class JNSlider extends JWheelSlider {

		public JNSlider(JBinomial parent, int n) {

			super(JSlider.VERTICAL, 0, 100, n);
			setMajorTickSpacing(10);
			setMinorTickSpacing(5);
			setPaintTicks(true);
			setFont( fira.font22 );
			setPaintLabels(true);
			addChangeListener(parent);

		}

	}


 
}
